package cn.yyxx.commsdk.core.permission;

import android.app.Activity;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;

import cn.yyxx.commsdk.base.entity.ResultInfo;
import cn.yyxx.commsdk.base.internal.ICallback;
import cn.yyxx.commsdk.base.internal.IRequestCallback;
import cn.yyxx.commsdk.base.utils.Logger;
import cn.yyxx.commsdk.core.constants.Constants;
import cn.yyxx.commsdk.core.network.SdkRequest;
import cn.yyxx.commsdk.core.ui.dialog.PrivacyPolicyDialog;
import cn.yyxx.commsdk.core.utils.ManifestUtils;
import cn.yyxx.commsdk.core.utils.SharedPreferencesUtils;
import cn.yyxx.support.JsonUtils;
import cn.yyxx.support.permission.IPermissionCallback;
import cn.yyxx.support.permission.Permission;
import cn.yyxx.support.permission.PermissionKit;

public class PermissionManager {
    public static PermissionManager permissionManager = null;
    private PermissionManager(){}
    public static PermissionManager getInstance(){
        if (permissionManager == null){
            permissionManager = new PermissionManager();
        }
        return permissionManager;
    }
    public void initAgreement(Activity activity, ICallback callback){


        String isShowPolicy = SharedPreferencesUtils.getSharedPreferences(activity, Constants.SP_KEY,"isShowPolicy");

        if (isShowPolicy.isEmpty()){
            SdkRequest.getInstance().initAgreement(activity, new IRequestCallback() {
                @Override
                public void onResponse(ResultInfo resultInfo) {
                    String policy_url = "";

                    try {
                        if (JsonUtils.hasJsonKey(new JSONObject(resultInfo.data),"agreement_url")){

                            JSONObject jsonObject = new JSONObject(resultInfo.data);
                            policy_url = jsonObject.getString("agreement_url");

                        }

                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    if (policy_url.isEmpty()){
                        callback.onResult(Constants.CODE_SUCCESS,"Dont's show Policy");
                        return;
                    }

                    new PrivacyPolicyDialog(activity, policy_url, new PrivacyPolicyDialog.OnClosePageListener() {
                        @Override
                        public void onUserAgreePolicy() {
                            SharedPreferencesUtils.setSharedPreferences( Constants.SP_KEY,"isShowPolicy","1",activity);
                            requestPermission(activity,callback);
                        }

                        @Override
                        public void onUserDisagreePolicy() {
                            callback.onResult(Constants.CODE_FAIL,"User disagree policy");
                        }
                    }).show();
                }
            });

        }else{
            requestPermission(activity,callback);
        }



    }
    void requestPermission(Activity activity,ICallback callback){

        if (!ManifestUtils.getMetaDataValue(activity,"yyxx.SkipPermission").isEmpty()){
            callback.onResult(Constants.CODE_SUCCESS,"User agree policy");
        }else if (!SharedPreferencesUtils.getSharedPreferences(activity,Constants.SP_KEY,"yyxxStopRequestPermission").isEmpty()){
            callback.onResult(Constants.CODE_SUCCESS,"User agree policy");
        }else{
            goRequestPermission(activity, new PermissonCallBack() {
                @Override
                public void onResult(Result code) {
                    if (code == Result.FINISH){
                        callback.onResult(Constants.CODE_SUCCESS,"User agree policy");
                    }
                }
            });
        }

    }


    void goRequestPermission(Activity activity,PermissonCallBack permissonCallBack) {
        PermissionKit.enablePermissionDialog = false;
        PermissionKit.with()
                .permission(Permission.READ_PHONE_STATE)
                .permission(Permission.Group.STORAGE)
                .request(activity, new IPermissionCallback() {
                    @Override
                    public void onGranted(List<String> list, boolean all) {
                        permissonCallBack.onResult(Result.GRANT);
                    }

                    @Override
                    public void onDenied(List<String> list, boolean never) {
                        permissonCallBack.onResult(Result.DENIED);
                    }

                    @Override
                    public void onProxyFinish() {

                        permissonCallBack.onResult(Result.FINISH);

                    }
                });
    }

    public interface PermissonCallBack{
        void onResult(Result code);
    }

    public enum Result{
        /**
         * 授权
         */
        GRANT,
        /**
         * 拒绝
         */
        DENIED,
        /**
         * 跳过
         */
        SKIP,
        /**
         * 首次拒绝，第二次不再申请
         */
        STOP,
        FINISH
    }
}
